@@ -14,6 +14,8 @@
 #
 # See the README file for information on usage and redistribution.
 #
+from __future__ import annotations
+
 import functools
 
 
@@ -28,13 +30,14 @@ class MultibandFilter(Filter):
 class BuiltinFilter(MultibandFilter):
     def filter(self, image):
         if image.mode == "P":
-            raise ValueError("cannot filter palette images")
+            msg = "cannot filter palette images"
+            raise ValueError(msg)
         return image.filter(*self.filterargs)
 
 
 class Kernel(BuiltinFilter):
     """
-    Create a convolution kernel.  The current version only
+    Create a convolution kernel. The current version only
     supports 3x3 and 5x5 integer and floating point kernels.
 
     In the current version, kernels can only be applied to
@@ -42,9 +45,10 @@ class Kernel(BuiltinFilter):
 
     :param size: Kernel size, given as (width, height). In the current
                     version, this must be (3,3) or (5,5).
-    :param kernel: A sequence containing kernel weights.
+    :param kernel: A sequence containing kernel weights. The kernel will
+                   be flipped vertically before being applied to the image.
     :param scale: Scale factor. If given, the result for each pixel is
-                    divided by this value.  The default is the sum of the
+                    divided by this value. The default is the sum of the
                     kernel weights.
     :param offset: Offset. If given, this value is added to the result,
                     after it has been divided by the scale factor.
@@ -57,7 +61,8 @@ class Kernel(BuiltinFilter):
             # default scale is sum of kernel
             scale = functools.reduce(lambda a, b: a + b, kernel)
         if size[0] * size[1] != len(kernel):
-            raise ValueError("not enough coefficients in kernel")
+            msg = "not enough coefficients in kernel"
+            raise ValueError(msg)
         self.filterargs = size, scale, offset, kernel
 
 
@@ -80,7 +85,8 @@ class RankFilter(Filter):
 
     def filter(self, image):
         if image.mode == "P":
-            raise ValueError("cannot filter palette images")
+            msg = "cannot filter palette images"
+            raise ValueError(msg)
         image = image.expand(self.size // 2, self.size // 2)
         return image.rankfilter(self.size, self.rank)
 
@@ -153,7 +159,8 @@ class GaussianBlur(MultibandFilter):
     approximates a Gaussian kernel. For details on accuracy see
     <https://www.mia.uni-saarland.de/Publications/gwosdek-ssvm11.pdf>
 
-    :param radius: Standard deviation of the Gaussian kernel.
+    :param radius: Standard deviation of the Gaussian kernel. Either a sequence of two
+                   numbers for x and y, or a single number for both.
     """
 
     name = "GaussianBlur"
@@ -162,7 +169,12 @@ class GaussianBlur(MultibandFilter):
         self.radius = radius
 
     def filter(self, image):
-        return image.gaussian_blur(self.radius)
+        xy = self.radius
+        if not isinstance(xy, (tuple, list)):
+            xy = (xy, xy)
+        if xy == (0, 0):
+            return image.copy()
+        return image.gaussian_blur(xy)
 
 
 class BoxBlur(MultibandFilter):
@@ -172,18 +184,31 @@ class BoxBlur(MultibandFilter):
     which runs in linear time relative to the size of the image
     for any radius value.
 
-    :param radius: Size of the box in one direction. Radius 0 does not blur,
-                   returns an identical image. Radius 1 takes 1 pixel
-                   in each direction, i.e. 9 pixels in total.
+    :param radius: Size of the box in a direction. Either a sequence of two numbers for
+                   x and y, or a single number for both.
+
+                   Radius 0 does not blur, returns an identical image.
+                   Radius 1 takes 1 pixel in each direction, i.e. 9 pixels in total.
     """
 
     name = "BoxBlur"
 
     def __init__(self, radius):
+        xy = radius
+        if not isinstance(xy, (tuple, list)):
+            xy = (xy, xy)
+        if xy[0] < 0 or xy[1] < 0:
+            msg = "radius must be >= 0"
+            raise ValueError(msg)
         self.radius = radius
 
     def filter(self, image):
-        return image.box_blur(self.radius)
+        xy = self.radius
+        if not isinstance(xy, (tuple, list)):
+            xy = (xy, xy)
+        if xy == (0, 0):
+            return image.copy()
+        return image.box_blur(xy)
 
 
 class UnsharpMask(MultibandFilter):
@@ -199,7 +224,7 @@ class UnsharpMask(MultibandFilter):
 
     .. _digital unsharp masking: https://en.wikipedia.org/wiki/Unsharp_masking#Digital_unsharp_masking
 
-    """  # noqa: E501
+    """
 
     name = "UnsharpMask"
 
@@ -355,7 +380,8 @@ class Color3DLUT(MultibandFilter):
 
     def __init__(self, size, table, channels=3, target_mode=None, **kwargs):
         if channels not in (3, 4):
-            raise ValueError("Only 3 or 4 output channels are supported")
+            msg = "Only 3 or 4 output channels are supported"
+            raise ValueError(msg)
         self.size = size = self._check_size(size)
         self.channels = channels
         self.mode = target_mode
@@ -370,7 +396,7 @@ class Color3DLUT(MultibandFilter):
         if hasattr(table, "shape"):
             try:
                 import numpy
-            except ImportError:  # pragma: no cover
+            except ImportError:
                 pass
 
         if numpy and isinstance(table, numpy.ndarray):
@@ -395,19 +421,21 @@ class Color3DLUT(MultibandFilter):
                 table, raw_table = [], table
                 for pixel in raw_table:
                     if len(pixel) != channels:
-                        raise ValueError(
+                        msg = (
                             "The elements of the table should "
-                            "have a length of {}.".format(channels)
+                            f"have a length of {channels}."
                         )
+                        raise ValueError(msg)
                     table.extend(pixel)
 
         if wrong_size or len(table) != items * channels:
-            raise ValueError(
+            msg = (
                 "The table should have either channels * size**3 float items "
                 "or size**3 items of channels-sized tuples with floats. "
                 f"Table should be: {channels}x{size[0]}x{size[1]}x{size[2]}. "
                 f"Actual length: {len(table)}"
             )
+            raise ValueError(msg)
         self.table = table
 
     @staticmethod
@@ -415,15 +443,15 @@ class Color3DLUT(MultibandFilter):
         try:
             _, _, _ = size
         except ValueError as e:
-            raise ValueError(
-                "Size should be either an integer or a tuple of three integers."
-            ) from e
+            msg = "Size should be either an integer or a tuple of three integers."
+            raise ValueError(msg) from e
         except TypeError:
             size = (size, size, size)
         size = [int(x) for x in size]
-        for size1D in size:
-            if not 2 <= size1D <= 65:
-                raise ValueError("Size should be in [2, 65] range.")
+        for size_1d in size:
+            if not 2 <= size_1d <= 65:
+                msg = "Size should be in [2, 65] range."
+                raise ValueError(msg)
         return size
 
     @classmethod
@@ -439,22 +467,23 @@ class Color3DLUT(MultibandFilter):
         :param target_mode: Passed to the constructor of the resulting
                             lookup table.
         """
-        size1D, size2D, size3D = cls._check_size(size)
+        size_1d, size_2d, size_3d = cls._check_size(size)
         if channels not in (3, 4):
-            raise ValueError("Only 3 or 4 output channels are supported")
+            msg = "Only 3 or 4 output channels are supported"
+            raise ValueError(msg)
 
-        table = [0] * (size1D * size2D * size3D * channels)
+        table = [0] * (size_1d * size_2d * size_3d * channels)
         idx_out = 0
-        for b in range(size3D):
-            for g in range(size2D):
-                for r in range(size1D):
+        for b in range(size_3d):
+            for g in range(size_2d):
+                for r in range(size_1d):
                     table[idx_out : idx_out + channels] = callback(
-                        r / (size1D - 1), g / (size2D - 1), b / (size3D - 1)
+                        r / (size_1d - 1), g / (size_2d - 1), b / (size_3d - 1)
                     )
                     idx_out += channels
 
         return cls(
-            (size1D, size2D, size3D),
+            (size_1d, size_2d, size_3d),
             table,
             channels=channels,
             target_mode=target_mode,
@@ -481,23 +510,24 @@ class Color3DLUT(MultibandFilter):
                             lookup table.
         """
         if channels not in (None, 3, 4):
-            raise ValueError("Only 3 or 4 output channels are supported")
+            msg = "Only 3 or 4 output channels are supported"
+            raise ValueError(msg)
         ch_in = self.channels
         ch_out = channels or ch_in
-        size1D, size2D, size3D = self.size
+        size_1d, size_2d, size_3d = self.size
 
-        table = [0] * (size1D * size2D * size3D * ch_out)
+        table = [0] * (size_1d * size_2d * size_3d * ch_out)
         idx_in = 0
         idx_out = 0
-        for b in range(size3D):
-            for g in range(size2D):
-                for r in range(size1D):
+        for b in range(size_3d):
+            for g in range(size_2d):
+                for r in range(size_1d):
                     values = self.table[idx_in : idx_in + ch_in]
                     if with_normals:
                         values = callback(
-                            r / (size1D - 1),
-                            g / (size2D - 1),
-                            b / (size3D - 1),
+                            r / (size_1d - 1),
+                            g / (size_2d - 1),
+                            b / (size_3d - 1),
                             *values,
                         )
                     else:
